#!/usr/bin/perl
# $Revision: 1.10 $
# Copyright 2009 Teleflora
# 
# Setup a "parallel" SSHd service which only permits 2-factor authentication.
# PA-DSS 11.2
# PCI-DSS 8.2, 8.3
#

use strict;
use warnings;
use Getopt::Long;

my $VERSION = 0;
my $HELP = 0;
my $SETUP =0;
my $START = 0;
my $STOP = 0;
my $STATUS = 0;
my $ENABLE = 0;
my $DISABLE = 0;

GetOptions(
	"version" => \$VERSION,
	"help" => \$HELP,
	"setup" => \$SETUP,
	"start" => \$START,
	"stop" => \$STOP,
	"status" => \$STATUS,
	"enable" => \$ENABLE,
	"disable" => \$DISABLE,
);



# --help
if($HELP != 0) {
	usage();
	exit(0);
}

# --version
if($VERSION != 0) {
	print('$Revision: 1.10 $' . "\n");
	exit(0);
}

# --status
if($STATUS != 0) {
	exit(tfremote_status());
}

############
# Root is needed to execute commands below.
############
if($< != 0) {
	usage();
	print("Error: Must run this script as 'sudo' or root.\n");
	exit(1);
}


# --start
if($START != 0) {
	exit(start_tfremote());
}

# --stop
if($STOP != 0) {
	exit(stop_tfremote());
}


# --enable
if($ENABLE != 0) {
	exit(enable_tfremote());
}

# --disable
if($DISABLE != 0) {
	exit(disable_tfremote());
}


# --setup
if($SETUP != 0) {
	exit(setup_tfremote());
}



usage();
exit(0);
#####################################################################
#####################################################################
#####################################################################


sub usage
{
	print("Usage:\n");
	print("$0 --help\n");
	print("$0 --version\n");
	print("$0 --start\n");
	print("$0 --stop\n");
	print("$0 --status\n");
	print("$0 --setup\n");
	print("$0 --enable\n");
	print("$0 --disable\n");
	print("\n");
	print("TFRemote is a system service which only allows incoming connections authenticated\n");
	print("via ssh public key encryption. As the service is actually a second instance of OpenSSH,\n");
	print("all configuration and usage rules which apply to SSH, apply here as well.\n");
	print("This script is capable of setting up the 'tfremote' service\n");
	print("on a redhat machine, as well as enabling and disabling the service.\n");
	print("\n");
	return(0);
}


sub start_tfremote
{
	system("/sbin/service tfremote start");
	return($?);
}


sub stop_tfremote
{
	system("/sbin/service tfremote stop");
	return($?);
}


sub tfremote_status
{
	system("/sbin/service tfremote status");
	return($?);
}


sub enable_tfremote
{
	my $prog = "";

	foreach $prog ("/sbin/chkconfig", "/usr/sbin/chkconfig") {
		next if(! -f "$prog");
		system("$prog --add tfremote");
		system("$prog --level 3 tfremote on");
		system("$prog --level 5 tfremote on");
		last;
	}

	return($?);
}


sub disable_tfremote
{
	my $prog = "";

	foreach $prog ("/sbin/chkconfig", "/usr/sbin/chkconfig") {
		next if(! -f "$prog");
		system("$prog --level 3 tfremote off");
		system("$prog --level 5 tfremote off");
		last;
	}

	return($?);
}



#
# Create the system service.
# This is basically a clone of "sshd" which runs concurrent to sshd, uses the same config files, but is just renamed.
# The reason for a separate, concurrent process, is to enforce on a "system" level, the use of two-factor authentication.
#
sub setup_tfremote
{
	my $prog = "";
	my $file = "";
	my $string = "";


	# Verify our "source files" exist.
	foreach $file ("/etc/init.d/sshd", "/etc/ssh/sshd_config", "/usr/sbin/sshd") {
		if(! -f "$file") {
			print("Error: \"$file\". $!\n");
			return(1);
		}
	}

	foreach $file ("/etc/sysconfig", "/etc/ssh") {
		if(! -d "$file") {
			print("Error: \"$file\" $!\n");
			return(2);
		}
	}


	# /etc/init.d
	open(OLD, "< /etc/init.d/sshd");
	open(NEW, "> /etc/init.d/tfremote");
	while(<OLD>) {
		$string = $_;
		$string =~ s/ssh_host_key/tfremote_host_key/g;
		$string =~ s/ssh_host_rsa_key/tfremote_host_rsa_key/g;
		$string =~ s/ssh_host_dsa_key/tfremote_host_dsa_key/g;

		$string =~ s/ssh_random_seed/tfremote_random_seed/g;
		$string =~ s/sshd/tfremote/g;
		$string =~ s/prog=\"([[:print:]]+)\"/prog=\"tfremote\"/g;
		$string =~ s/description: OpenSSH server daemon/description:RTI Remote Access/g;

		print(NEW $string);
	}
	close(OLD);
	close(NEW);
	system("chmod --reference=/etc/init.d/sshd /etc/init.d/tfremote");
	system("chown --reference=/etc/init.d/sshd /etc/init.d/tfremote");


	mkdir("/var/empty/tfremote");
	system("chmod --reference=/var/empty/sshd /var/empty/tfremote");
	system("chown --reference=/var/empty/sshd /var/empty/tfremote");


	# Make the service start at bootup.
	# Must use a hard link here (not symlink), as, redhat init.d/functions "stop()" function will
	# wind up killing all "sshd" processes if we use a symlink.
	# Redhat updates should work fine in the event that we use hard links.
	unlink("/usr/sbin/tfremote");
	system("ln /usr/sbin/sshd /usr/sbin/tfremote");
	enable_tfremote();




	# tfremote_config
	open(OLD, "< /etc/ssh/sshd_config");
	open(NEW, ">  /etc/ssh/tfremote_config");
	while(<OLD>)
	{
		next if(/Port /i);
		next if(/Protocol /i);
		next if(/HostKey /i);
		next if(/SyslogFacility /i);
		next if(/LogLevel /i);
		next if(/PidFile /i);
		next if(/PermitRootLogin /i);
		next if(/RSAAuthentication /i);
		next if(/PubkeyAuthentication /i);
		next if(/RhostsRSAAutthenitcation /i);
		next if(/HostbasedAuthentication /i);
		next if(/IgnoreUserKnownHosts /i);
		next if(/IgnoreRhosts /i);
		next if(/PermitEmptyPasswords /i);
		next if(/PasswordAuthentication /i);
		next if(/KerberosAuthentication /i);
		next if(/GSSAPIAuthentication /i);
		next if(/UsePAM /i);
		next if(/Banner /i);

		print(NEW);
	}
	close(OLD);


	print(NEW "\n");
	print(NEW "\n");
	print(NEW "#\n"); 
	print(NEW "# PA-DSS Compliant Settings\n");
	print(NEW "# Setup by tfremote " . '$Revision: 1.10 $' . "\n");
	print(NEW "#\n"); 
	print(NEW "Port 15022\n");
	print(NEW "Protocol 2\n");
	print(NEW "HostKey /etc/ssh/tfremote_host_rsa_key\n");
	print(NEW "HostKey /etc/ssh/tfremote_host_dsa_key\n");
	print(NEW "SyslogFacility AUTH\n");
	print(NEW "LogLevel VERBOSE\n");
	print(NEW "PidFile /var/run/tfremote.pid\n");
	print(NEW "PermitRootLogin no\n");
	print(NEW "RSAAuthentication no\n");
	print(NEW "PubkeyAuthentication yes\n");
	print(NEW "RhostsRSAAuthentication no\n");
	print(NEW "HostBasedAuthentication no\n");
	print(NEW "IgnoreUserKnownHosts yes\n");
	print(NEW "IgnoreRhosts yes\n");
	print(NEW "PermitEmptyPasswords no\n");
	print(NEW "PasswordAuthentication no\n");
	print(NEW "KerberosAuthentication no\n");
	print(NEW "GSSAPIAuthentication no\n");
	print(NEW "UsePAM no\n");
	print(NEW "Banner /etc/motd\n");
	print(NEW "\n");
	print(NEW "\n");
	close(NEW);

	if(! -f "/etc/ssh/tfremote_config") {
		print("Error: /etc/ssh/tfremote_config  $!\n");
		return(5);
	}


	# Here is where we tell 'sshd' to read our PA-DSS compliant settings.
	open(NEW, "> /etc/sysconfig/tfremote");
	print(NEW "OPTIONS=\"-f /etc/ssh/tfremote_config\"\n");
	close(NEW);

	if(! -f "/etc/sysconfig/tfremote") {
		print("Error: /etc/sysconfig/tfremote $!\n");
		return(6);
	}
}
